/*
  Funcin Justify() para justificar a una longitud de lnea concreta
  un fichero de texto

  Parmetros:
  -----------
  cFile .................... Nombre del fichero a justificar
  nLinea ................... Longitud de lnea deseada

*/


#include "FileIO.Ch"
#define  CRLF         Chr(13)+Chr(10)    // Fin de lnea
#define  NLONG        256                // tamao de lnea mxima
#define  BUFFER       (10*1024)          // Buffer para FLocate()


STATIC nHandle        // Handle del fichero a justificar
STATIC nHandleW       // Handle para el fichero de salida

FUNCTION Justify(cFile,nLinea)
   LOCAL nPos, nPosIni, nPosFin, cAuxBuffer, nEof, cBuffer := "",;
         cNuevaLinea, nBoj := 0, nEoj := 0, nAvanzado := 0
   LOCAL nPosPoint
   LOCAL cFileFus
   LOCAL lValRet := 0
   LOCAL nCont
   LOCAL cBuffParrafo
   LOCAL nStart

   // Abre el fichero si existe, si no devuelve el error 1
   IF File(cFile)
      nHandle   := FOpen(cFile, FO_READ)

      // Esto crea y/o abre el fichero de salida
      nPosPoint := AT(".",cFile)
      cFileFus  := SubStr(cFile,1,nPosPoint-1)+".prn"
      nHandleW := FCreate(cFileFus)

    ELSE
      lValRet := 1
      RETURN lValRet
   END IF

   // Si no se le pasa el parmetro de la longitud de lnea, toma por
   // defecto 76
   IF ValType(nLinea) # "N"
      nLinea := 76
   END IF

   nEof := FSEEK(nHandle, 0, FS_END)
   FSEEK(nHandle, 0)

   nPos       := 1
   nPosIni    := 0
   nPosFin    := 1

   // Proceso hasta fin de fichero
   WHILE nPosIni < nEof

      cBuffer    := Space(NLONG)
      cAuxBuffer := Space(NLONG)

      FRead(nHandle,@cAuxBuffer,NLONG)
      nPos := At(CRLF,cAuxBuffer)

      // Fuerza la terminacin si encuentra una marca de fin de fichero
      // o una lnea totalmente vaca (incluso sin marcas de formato)
      IF AllTrim(cAuxBuffer) == Chr(26) .OR. Empty(AllTrim(cAuxBuffer))
         EXIT
      END IF

      // Controla terminaciones de lnea cin CRLF
      IF nPos == 0
         IF !Empty(AllTrim(cAuxBuffer))
            nPos := At(Chr(26),cAuxBuffer)          // con Chr(26)
            IF nPos == 0
               IF !Len(AllTrim(cAuxBuffer)) == 0
                  nPos := Len(AllTrim(cAuxBuffer))+3  // incluso sin ello
               END IF
            END IF
         END IF
      END IF

      nPosFin += nPos
      FSeek(nHandle, nPosIni, FS_SET)
      FRead(nHandle,@cBuffer,nPosFin-nPosIni)

      // Una vez hallada una lnea, la guarda en un array para tratarla
      // despus
      IF Trim(cBuffer) # "^j"
         FWrite(nHandleW, Trim(cBuffer)+CRLF)
       ELSE

         //determina principio (nBoj) y fin (nEoj) de justificacin
         //procesa todo un rea de texto comprendida entre ambos
         nBoj := nPosIni
         FSeek(nHandle, nAvanzado, FS_SET)
         FLocate(nHandle, "^fj")
         nAvanzado := FPos(nHandle)
         nEoj += nAvanzado
         nAvanzado++
         FSeek(nHandle, nBoj+2, FS_SET)
         nStart := nBoj+2
         WHILE nStart < nEoj
            cBuffParrafo := Space(nEoj-nStart)
            FSeek(nHandle, nStart, FS_SET)
            FRead(nHandle, @cBuffParrafo, nEoj-nStart)
            FWrite(nHandleW, TrataParrafo(cBuffParrafo,nLinea)+CRLF+CRLF)
            nStart := FPos(nHandle)+1
            nPosFin := nStart+4
         END WHILE
      END IF

      FSeek(nHandle, nPosFin+1, FS_SET)
      nPosIni    := nPosFin+2

   END WHILE

   FClose(nHandle)           // Cierra fichero de entrada
   FClose(nHandleW)          // Cierra fichero de salida

RETURN lValRet


// Esta funcin va formateando cada uno de los prrafos justificables
// hallados en el texto
// Sus parmetros son:
//
// cBuffer: el prrafo
// nLinea:  margen derecho
//
//
STATIC FUNCTION TrataParrafo(cBuffer,nLinea)
   LOCAL cAuxParrafo
   LOCAL nLongParrafo
   LOCAL nLeidos
   LOCAL nBlanco
   LOCAL cLinea1
   LOCAL cLinea2

   cAuxParrafo := StrTran(cBuffer, CRLF, " ")  // quita retornos de carro 
   nLongParrafo  := Len(cAuxParrafo)
   IF nLongParrafo > nLinea
      nBlanco   := 1
      nLeidos   := 1
       
      // Este bucle va poniendo los retornos de nuevo segn el tamao del
      // margen derecho
      WHILE nLongParrafo > nLeidos
         nBlanco := Rat(" ", SubStr(cAuxParrafo,nLeidos, nLinea))
         nLeidos := nLeidos + nBlanco
         cLinea1 := AllTrim(SubStr(cAuxParrafo, 1, nLeidos-1))
         cLinea2 := TrataLinea(cLinea1, nLinea) + CRLF
         cAuxParrafo := cLinea2 + AllTrim(SubStr(cAuxParrafo, nLeidos))
         nLeidos += Len(cLinea2) - Len(cLinea1)
         IF Len(AllTrim(SubStr(cAuxParrafo, nLeidos))) < nLinea
            EXIT
         END IF
      END WHILE

   END IF

RETURN cAuxParrafo


// Esta funcin justifica cada una de las lneas, antes de escribirlas
// en el fichero de salida
// Sus parmetros son:
//
// cLinea:   Lnea a justificar
// nLinea:   Margen derecho
//
//
STATIC FUNCTION TrataLinea(cLinea, nLinea)
   LOCAL nBlancos
   LOCAL nAjuste
   LOCAL nPosJust
   LOCAL nCont, nCont2
   LOCAL cAuxLinea
   LOCAL nLastCR

   nLastCR := Rat(CRLF, cLinea)
   IF nLastCR # 0
      cAuxLinea := SubStr(cLinea,nLastCR+2)
    ELSE
      cAuxLinea := cLinea
   END IF

   nBlancos := StrCount(cAuxLinea, " ")
   nAjuste  := nLinea - Len(cAuxLinea)
   nPosJust := nLinea

   // Si la lnea tiene ms blancos de los que se necesitan para el ajuste
   // inserta blancos en las primeras posiciones por la derecha
   IF nAjuste <= nBlancos
      FOR nCont := 1 TO nAjuste
          nPosJust  :=  Rat(" ", SubStr(cAuxLinea,1, nPosJust-1))
          cAuxLinea :=  SubStr(cAuxLinea,1,nPosJust)+;
                               " " + SubStr(cAuxLinea,nPosJust+1)
      END FOR
      cLinea := SubStr(cLinea,1,nLastCR-1) +;
                If(nLastCR # 0, CRLF, "") + cAuxLinea

    // En caso contrario inserta blancos en todos los posibles y
    // cuando no le quedan ms vuelve a inserta en los mismos
    // espacios
    ELSE
      FOR nCont := 1 TO nBlancos
          nPosJust  :=  Rat(" ", SubStr(cAuxLinea,1, nPosJust-1))
          cAuxLinea :=  SubStr(cAuxLinea,1,nPosJust)+;
                               " " + SubStr(cAuxLinea,nPosJust+1)
      END FOR
      IF nAjuste > nCont
         FOR nCont2 := 1 TO nAjuste - nCont + 1
             cAuxLinea :=  SubStr(cAuxLinea,1,nPosJust)+;
                           " " + SubStr(cAuxLinea,nPosJust+1)

         END FOR
      ENDIF

      cLinea := SubStr(cLinea,1,nLastCR-1) +;
                If(nLastCR # 0, CRLF, "") + cAuxLinea

   END IF

RETURN cLinea


