/*
  Funcin Formato() para analizar un fichero de texto conteniendo
  marcas para fusionar. Busca lnea a lnea las marcas y las
  sustituye por sus valores.

  Parmetros:
  -----------
  cFile .................... Nombre del fichero a analizar

*/


#include "FileIO.Ch"
#define  CRLF         Chr(13)+Chr(10)     // Fin de lnea
#define  NLONG        256                 // tamao de lnea mxima

STATIC nHandle        // Handle del fichero a analizar
STATIC nHandleW       // Handle para el fichero de salida

FUNCTION Merge(cFile)
   LOCAL nPos, nPosIni, nPosFin, cAuxBuffer, nEof, cBuffer := "",;
         cNuevaLinea
   LOCAL nPosPoint
   LOCAL cFileFus
   LOCAL lValRet := 0

   // Abre el fichero si existe, si no devuelve el error 1
   IF File(cFile)
      nHandle   := FOpen(cFile, FO_READ)

      // Esto crea y abre el fichero de salida
      nPosPoint := AT(".",cFile)
      cFileFus  := SubStr(cFile,1,nPosPoint-1)+".fus"
      nHandleW  := FCreate(cFileFus)

    ELSE
      lValRet := 1
      RETURN lValRet
   END IF

   nEof := FSEEK(nHandle, 0, FS_END)
   FSEEK(nHandle, 0)

   nPos       := 1
   nPosIni    := 0
   nPosFin    := 1

   // Procesa las lneas mientras mo es final de fichero
   WHILE nPosIni < nEof

      cBuffer    := Space(NLONG)
      cAuxBuffer := Space(NLONG)

      FRead(nHandle,@cAuxBuffer,NLONG)
      nPos := At(CRLF,cAuxBuffer)

      // Fuerza la terminacin si encuentra una marca de fin de fichero
      // o una lnea totalmente vaca (incluso sin marcas de formato)
      IF AllTrim(cAuxBuffer) == Chr(26) .OR. Empty(AllTrim(cAuxBuffer))
         EXIT
      END IF

      // Controla terminaciones de lnea sin CRLF
      IF nPos == 0
         IF !Empty(AllTrim(cAuxBuffer))
            nPos := At(Chr(26),cAuxBuffer)          // con Chr(26)
            IF nPos == 0
               IF !Len(AllTrim(cAuxBuffer)) == 0
                  nPos := Len(AllTrim(cAuxBuffer))+3  // incluso sin ello
               END IF
            END IF
         END IF
      END IF

      nPosFin += nPos
      FSeek(nHandle, nPosIni, FS_SET)
      FRead(nHandle,@cBuffer,nPosFin-nPosIni)

      // Una vez hallada una lnea, la trata para realizar en ella
      // las sustituciones programadas
      cNuevaLinea := TrataLinea(Trim(cBuffer))

      // Una vez la lnea con sus sustituciones la vuelve a escribir
      FWrite(nHandleW,cNuevaLinea)

      FSeek(nHandle, nPosFin+1, FS_SET)
      nPosIni    := nPosFin

   END WHILE

   FWrite(nHandleW,CRLF)     // Escribe CRLF al final del fichero

   FClose(nHandle)           // Cierra fichero de entrada
   FClose(nHandleW)          // Cierra fichero de salida

RETURN lValRet


// Funcin para tratar cada una de las lneas y sustiuir en ellas
// sus marcas {} de principio y fin de fusin por los elementos
// fusionados
STATIC FUNCTION TrataLinea(cBuffer)
   LOCAL StartExp     := 1
   LOCAL EndExp       := 1
   LOCAL nCurPos      := 0
   LOCAL xExp
   LOCAL aExp         := {}
   LOCAL nPos
   LOCAL cNuevaLinea  := ""

   StartExp := At("{", cBuffer)
   EndExp   := At("}", cBuffer)

   IF StartExp # 0
      xExp   := SubStr(cBuffer,StartExp+1,EndExp-StartExp-1)
      nCurPos := EndExp+1
   END IF

   // Evala el primer hallazgo y guarda en un array, su valor y sus
   // posiciones de comienzo y fin en la lnea en cuestin
   IF xExp # NIL
      xExp := EVAL(&("{||"+xExp+"}"))
      xExp := If(ValType(xExp) == "N",Str(xExp), xExp)
      AADD(aExp, {AllTrim(xExp), StartExp, EndExp})
   END IF

   WHILE nCurPos # 0
     EndExp++
     StartExp := EndExp + At("{",SubsTr(cBuffer, EndExp))
     nCurPos  := At("}",SubsTr(cBuffer, EndExp))
     EndExp   += nCurPos

     IF nCurPos # 0
        xExp := SubStr(cBuffer,StartExp,EndExp-StartExp-1)

        // Evala los hallazgos sucesivos y los guarda en el mismo array
        // donde lo hizo con el primero
        IF xExp # NIL
           xExp := EVAL(&("{||"+xExp+"}"))
           xExp := If(ValType(xExp) == "N",Str(xExp), xExp)
           AADD(aExp, {AllTrim(xExp), StartExp, EndExp})
        END IF

      ELSE
        EXIT
     END IF

   END WHILE


   // Construye las nuevas lneas segn la informacin contenida en
   // el array
   IF LEN(aExp) == 0
      cNuevaLinea := cBuffer
    ELSEIF LEN(aExp) == 1
       cNuevaLinea := SubStr(cBuffer,1,aExp[1,2]-1)+aExp[1,1] +;
                      SubStr(cBuffer,aExp[1,3]+1)
    ELSEIF LEN(aExp) > 1
      FOR nPos := 1 TO LEN(aExp)
          cNuevaLinea := SubStr(cBuffer,1,aExp[nPos,2]-2)+aExp[nPos,1] +;
                         SubStr(cBuffer,aExp[nPos,3]+1)
          cNuevaLinea := TrataLinea(cNuevaLinea)
      END FOR
   END IF

RETURN cNuevaLinea

